from enigma import iPlayableService, iServiceInformation
from ServiceReference import ServiceReference
from Plugins.Plugin import PluginDescriptor
from Components.config import config, getConfigListEntry, ConfigSelection, ConfigSubsection
from Screens.Screen import Screen
from Components.ActionMap import ActionMap
from Components.Button import Button
from Components.Label import Label
from Components.ConfigList import ConfigList, ConfigListScreen
from Components.ServiceEventTracker import ServiceEventTracker
import string
# depends on Videomode Plugin
from Plugins.SystemPlugins.Videomode.VideoHardware import video_hw

session = [ ]

reschoices = [("1080i", _("1080i")), ("720p", _("720p")), ("576i", _("576i")), \
	("576p", _("576p")), ("480i", _("480i")), ("480p", _("480p"))]
config.plugins.autoresolution = ConfigSubsection()
config.plugins.autoresolution.i1080 = ConfigSelection(default = "1080i", choices = reschoices)
config.plugins.autoresolution.i576 = ConfigSelection(default = "576i", choices = reschoices)
config.plugins.autoresolution.p720 = ConfigSelection(default = "720p", choices = reschoices)
config.plugins.autoresolution.i480 = ConfigSelection(default = "480i", choices = reschoices)

def build_switchdic():
	dic = { '1080': config.plugins.autoresolution.i1080.value, \
		'576':config.plugins.autoresolution.i576.value, \
		'480':config.plugins.autoresolution.i480.value, \
		'720':config.plugins.autoresolution.p720.value}
	return dic
switchdic = build_switchdic()
		
def get_PreferredModes():
	try:
		modes = open("/proc/stb/video/videomode_preferred").read()[:-1]
		modes_preferred = modes.split(' ')
	except IOError:
		modes_preferred = self.get_AvailableModes()
	return modes_preferred
preferedmodes = get_PreferredModes()	


class AutoResMenu(Screen, ConfigListScreen):
	skin = """
		<screen position="150,100" size="420,315" title="Auto Resolution" >
		<widget name="txt" position="10,10" size="380,20" text="Choose Content Resolution:" font="Regular;18" halign="left" transparent="1" />
		<widget name="config" position="10,40" size="380,100" scrollbarMode="showOnDemand" />
		<widget name="availRes" position="10,150" size="380,140" font="Regular;16" halign="left" transparent="1" />
		<ePixmap position="260,265" zPosition="1" size="140,40" pixmap="skin_default/buttons/green.png" alphatest="on" />
		<ePixmap position="10,265" zPosition="1" size="140,40" pixmap="skin_default/buttons/red.png" alphatest="on" />
		<widget name="key_red" position="10,265" zPosition="2" size="140,40" halign="center" valign="center" font="Regular;21" transparent="1" shadowColor="black" shadowOffset="-1,-1" />
		<widget name="key_green" position="260,265" zPosition="2" size="140,40" halign="center" valign="center" font="Regular;21" transparent="1" shadowColor="black" shadowOffset="-1,-1" />
		</screen>"""

	def __init__(self, session):
		self.session = session
		self.list = []
		Screen.__init__(self, session)
		ConfigListScreen.__init__(self, self.list)
		self["key_red"] = Button(_("Cancel"))
		self["key_green"] = Button(_("Ok"))
		self["txt"] = Label()
		pref_modes = string.join(preferedmodes, ' ')
		self["availRes"] = Label(_('TV supports this Resolution:\n\n%s ' % pref_modes))
		self.createScreen()
		
		self["setupActions"] = ActionMap(["SetupActions", "ColorActions"],
		{
			"cancel": self.close,
			"red": self.close,
			"green": self.go,
			"ok": self.go
		}, -2)
		
	def createScreen(self):
		self.list = []
		self.list.append(getConfigListEntry(_("1080i Content"), config.plugins.autoresolution.i1080))
		self.list.append(getConfigListEntry(_("720p Content"), config.plugins.autoresolution.p720))
		self.list.append(getConfigListEntry(_("576i Content"), config.plugins.autoresolution.i576))
		self.list.append(getConfigListEntry(_("480i Content"), config.plugins.autoresolution.i480))
		self["config"].list = self.list
		self["config"].setList(self.list)
	
	def go(self):
		for x in self["config"].list:
			x[1].save()
			global switchdic
			switchdic = build_switchdic()
		self.close()
			

class AutoRes(Screen):
	rates = video_hw.rates
	modes = video_hw.modes
	widescreen_modes = video_hw.widescreen_modes
	
	def __init__(self, session):
		Screen.__init__(self, session)
		self.tracker = ServiceEventTracker(screen=self, eventmap=
				{ iPlayableService.evVideoSizeChanged: self.nav_event, })
		self.height = ''
		
	def nav_event(self):
		height = self.getVideoSize()
		if height != self.height:
			self.height = height
			if switchdic.has_key(self.height):
				if switchdic[self.height] in preferedmodes:
					port = config.av.videoport.value
					mode = switchdic[self.height]
					rate = config.av.videorate[mode].value
					video_hw.setMode(port, mode, rate)
				else:
					print 'AutoRes: TV dont support %s' % switchdic[self.height]
	
	def getVideoSize(self):
		height = ''
		service = session.nav.getCurrentService()
		if service is not None:
			svinfo = service.info()
			if session.nav.getCurrentlyPlayingServiceReference() is not None:
				height = svinfo and svinfo.getInfo(iServiceInformation.sVideoHeight) or -1
		return str(height)
	
	

def autostart(reason, **kwargs):
	global session
	if "session" in kwargs:
		session = kwargs["session"]
		AutoRes(session)

def startSetup(menuid):
	if menuid != "system": 
		return [ ]
	return [(_("Autoresolution") + "...", autoresSetup, "autores_setup", 45)]

def autoresSetup(session, **kwargs):
	session.open(AutoResMenu)

def Plugins(path, **kwargs):
	return [PluginDescriptor(where = [PluginDescriptor.WHERE_SESSIONSTART], fnc = autostart), \
		PluginDescriptor(name=_("Autoresolution"), description=_("Autoresolution Switch"), where = PluginDescriptor.WHERE_MENU, fnc=startSetup) ]
